# -*- coding: utf-8 -*-

""" Module summary description.

More detailed description.
"""

GDAL_TO_NUMPY = {1: "int8",
                 2: "uint16",
                 3: "int16",
                 4: "uint32",
                 5: "int32",
                 6: "float32",
                 7: "float64",
                 10: "complex64",
                 11: "complex128"}
